function [f,Pxx,Pyy,Pxy,Cxy,C,Phxy,cxy,qxy,Lh] = spectra(varargin);

%
%  [f,Pxx,Pyy,Pxy,Cxy,C,Phxy,cxy,qxy,Lh] = spectra(L,x,y)
%  
%  Cross spectral analysis between timeseries x and y (must be of same
%  length. The output quantities are as follows:
%  
%   f    - frequencies
%   Pxx  - power spectrum of x
%   Pyy  - power spectrum of y
%   Pxy  - cross-spectrum of x and y
%   Cxy  - coherence (not squared) between x and y
%   C    - 5% significance level for coherence
%   Phxy - phase spectrum between x and y
%   cxy  - cospectrum between x and y
%   qxy  - quadspectrum between x and y
%   Lh   - effective length of window average
%  
%  The spectra are averaged with an L-point triangularly weighted window.
%  If L=-1 then the window will be defined as 10% of the length of the
%  series.
%  
%  If only x is specified then the program only outputs [f,Pxx]
%

% grab inputs, nargs = 2 is x only, nargs = 3 is x and y
[cax,args,nargs] = axescheck(varargin{:});
M = args{1};
x = args{2};
if nargs == 3
    y = args{3};
end

% check the sample length
n = length(x);
if nargs == 3
    if ( n ~= length(y) )
        ['Data sets x and y of different lengths!']
        f = []; Pxx = []; Pyy = []; Pxy = []; Cxy = []; C = []; Phxy = []; cxy = []; qxy = []; Lh = [];
        return;
    end
end

% set the smoothing window size
if ( M == -1 )
    L = floor(0.1*n);
else
    L = M;
end

% generate power spectra
[Pxx f] = spec(x);
if nargs == 3
    Pyy     = spec(y);
    Pxy     = xspec(x,y);
end
% smooth
[Pxx Lh] = wrunavg(Pxx, L, 1);
if nargs == 3
    Pyy      = wrunavg(Pyy, L, 1);
    Pxy_real = wrunavg(real(Pxy), L, 1);
    Pxy_imag = wrunavg(imag(Pxy), L, 2);
    Pxy      = Pxy_real + i*Pxy_imag;
    clear Pxy_real Pxy_imag
end
% scale to have integral(Pxx) = variance(x)
df = f(2)-f(1);
Pxx_scale = Pxx(1)*0.5*df + sum(Pxx(2:end-1))*df + Pxx(end)*0.5*df;
Pxx = Pxx .* var(x) ./ Pxx_scale;
if nargs == 3
    Pyy_scale = Pyy(1)*0.5*df + sum(Pyy(2:end-1))*df + Pyy(end)*0.5*df;
    Pyy = Pyy .* var(y) ./ Pyy_scale;
    Pxy = Pxy .* sqrt(var(x) ./ Pxx_scale) .* sqrt(var(y) ./ Pyy_scale);

    % set co- and quad-spectra
    cxy = real(Pxy);
    qxy = -imag(Pxy);

    % generate coherence and significance limit
    Cxy = abs(Pxy) ./ sqrt( Pxx .* Pyy );
    df = 2*Lh*length(x)/(2*length(Pxx));
    alpha = 0.05;
    C = 1 - alpha^(2./(df-2));
    % this is a very good approximation to:
    % F = finv(1-alpha,2,df-2); C = F./( 0.5*(df-2) + F);
    C = sqrt(C);

    % generate phase spectra and make points where Cxy < C NaNs
    Phxy = atan2( qxy , cxy );
    %ii = find( Cxy < C ); Phxy(ii) = NaN*ones(size(ii));
end
